package com.ksyun.util;


import com.fasterxml.jackson.core.type.TypeReference;
import com.google.common.collect.ImmutableMap;
import com.ksyun.constant.Consts;
import jodd.http.HttpRequest;
import jodd.http.HttpResponse;
import lombok.extern.slf4j.Slf4j;

import java.io.UnsupportedEncodingException;
import java.util.Map;

@Slf4j
public class HttpUtils<T> {

    public static <T> T postForm(String url, Map<String, String> params, Class<T> clazz) {
        HttpRequest request = HttpRequest
                .post(url)
                .accept("application/json")
                .contentType("application/x-www-form-urlencoded;charset=UTF-8");
        for (Map.Entry<String, String> entry : params.entrySet()) {
            request.form(entry.getKey(), entry.getValue());
        }
        String text = request.send().bodyText();
        log.info("result text:{}", text);
        text = reEncode(text);
        if (clazz.equals(String.class)) {
            return (T) text;
        }
        return UtilAll.fromJson(text, clazz);
    }

    public static <T> T postForm(String url, Map<String, String> params, TypeReference typeReference) {
        HttpRequest request = HttpRequest
                .post(url)
                .accept("application/json")
                .contentType("application/x-www-form-urlencoded;charset=utf8");
        for (Map.Entry<String, String> entry : params.entrySet()) {
            request.form(entry.getKey(), entry.getValue());
        }
        String text = request.send().bodyText();
        text = reEncode(text);
        return UtilAll.fromJson(text, typeReference);
    }

    public static <T> T get(String url, Map<String, String> params, Class<T> clazz) {
        HttpRequest request = HttpRequest.get(url).accept("application/json");
        request.query(params);
        HttpResponse response = request.send();
        String text = response.bodyText();
        log.info("result text:{}", text);
        text = reEncode(text);
        if (clazz.equals(String.class)) {
            return (T) text;
        }
        return UtilAll.fromJson(text, clazz);
    }

    public static <T> T get(String url, Map<String, String> params, TypeReference typeReference) {
        HttpRequest request = HttpRequest.get(url).accept("application/json");
        request.query(params);
        HttpResponse response = request.send();
        String text = response.bodyText();
        log.info("result text:{}", text);
        text = reEncode(text);
        return UtilAll.fromJson(text, typeReference);
    }

    private static String reEncode(String text) {
        try {
            return new String(text.getBytes("ISO-8859-1"), "UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error("编码解析错误,origin text:" + text, e);
            throw new RuntimeException("编码解析错误:" + text);
        }
    }
}
